﻿#include "service.h"
#include "parser.cpp.h"


/*---------------------------------------------------------
  Globals
---------------------------------------------------------*/
const wchar_t*  g_languageName              = L"PyLangPack";
const wchar_t*  g_languageFileExtensions[]  = { L".py",  NULL };
const CLSID     g_languageCLSID             = {0xDB1D5485, 0xD336, 0x4ED5, { 0xBC, 0x31, 0x13, 0x10, 0xC8, 0xB7, 0x40, 0x17 }};


const LanguageProperty g_languageProperties[] =
{
  { L"RequestStockColors", 0 },

  { L"ShowCompletion",     1     },
  { L"SortMemberList",     1     },

  { L"CodeSense",          1  },
  { L"CodeSenseDelay",     1500 },
  { L"MaxErrorMessages",   5     },
  { L"QuickInfo",          0     },
  { L"MatchBraces",        1     },
  { L"ShowMatchingBrace",  0     },
  { L"MatchBracesAtCaret", 0     },

  { NULL, 0 }
};

/*---------------------------------------------------------
  Create Service
---------------------------------------------------------*/
HRESULT CreateBabelService( out IBabelService** babelService )
{
  TRACE(L"CreateBabelService");
  OUTARG(babelService);

  *babelService = new Service();
  if (*babelService == NULL) return E_OUTOFMEMORY;

  return S_OK;
}

override const CommentFormat *Service::getCommentFormat() const
{
    // TODO: Change this definition using the comment formats of your language
	static CommentFormat commentFormat = { "#", NULL, NULL, true };
    return &commentFormat;
}


/*---------------------------------------------------------
  Tokens
---------------------------------------------------------*/
override const TokenInfo* Service::getTokenInfo() const
{
  	static TokenInfo tokenInfoTable[] =
	{
		//always end with the 'TokenEnd' token.
		//{ TokenEnd,     ClassText,      "<unknown>" }

		// Python Keywords and defines
		{ IDENTIFIER,		ClassIdentifier,    "identifier '%s'"	, CharIdentifier 	},
		{ NUMBER,			ClassNumber,        "number ('%s')"		, CharLiteral    	},
		{ STRING_QUOTE,		ClassString,		"Quote"				, CharLiteral	 	},


		{ KWIF,				ClassKeyword,       ""					, CharKeyword 		},		
		{ KWELSE,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWWHILE,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWFOR,			ClassKeyword,       ""					, CharKeyword 		},		
		{ KWAND,			ClassKeyword,       ""					, CharKeyword 		},		
		{ KWDEL,			ClassKeyword,       ""					, CharKeyword 		},		
		{ KWFROM,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWNOT,			ClassKeyword,       ""					, CharKeyword 		},		
		{ KWAS,				ClassKeyword,		""					, CharKeyword 		},		
		{ KWELIF,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWGLOBAL,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWOR,				ClassKeyword,       ""					, CharKeyword 		},		
		{ KWWITH,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWASSERT,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWPASS,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWYIELD,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWBREAK,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWEXCEPT,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWIMPORT,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWPRINT,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWCLASS,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWEXEC,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWIN,				ClassKeyword,       ""					, CharKeyword 		},		
		{ KWRAISE,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWCONTINUE,		ClassKeyword,       ""					, CharKeyword 		},
		{ KWFINALLY,		ClassKeyword,       ""					, CharKeyword 		},	
		{ KWIS,				ClassKeyword,       ""					, CharKeyword 		},		
		{ KWRETURN,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWDEF,			ClassKeyword,       ""					, CharKeyword 		},		
		{ KWLAMBDA,			ClassKeyword,       ""					, CharKeyword 		},	
		{ KWTRY,			ClassKeyword,       ""					, CharKeyword 		},	

		{ MFALSE,			ClassSymbol,        ""					, CharKeyword 		},	
		{ MTRUE,			ClassSymbol,        ""					, CharKeyword 		},	


		{ LEX_WHITE,		ClassText,          "white space"		, CharWhiteSpace	},
		{ LEX_LINE_COMMENT, ClassComment,		"comment"			, CharLineComment	},
		{ LEX_COMMENT,		ClassComment,       "comment"			, CharComment		},

		//Always end with the 'TokenEnd' token.
		{ TokenEnd,			ClassText,			"<unknown>" }
	};


  return tokenInfoTable;
};




